package io.hmit.modules.serviceOrder.controller;

import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.afterturn.easypoi.excel.entity.result.ExcelImportResult;
import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.exception.ErrorCode;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ConvertUtils;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.security.user.SecurityUser;
import io.hmit.modules.serviceOrder.dto.UserDTO;
import io.hmit.modules.serviceOrder.excel.UserExcel;
import io.hmit.modules.serviceOrder.service.RoleUserService;
import io.hmit.modules.serviceOrder.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;


/**
 * 用户
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-01-27
 */
@RestController
@RequestMapping("serviceOrder/user")
@Api(tags="用户")
public class UserController {
    @Autowired
    private UserService userService;
    @Autowired
    private RoleUserService roleUserService;
    @Value("${QR.download-path}")
    private String QRPath;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    @RequiresPermissions("serviceOrder:user:page")
    public Result<PageData<UserDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<UserDTO> page = userService.page(params);

        return new Result<PageData<UserDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @RequiresPermissions("serviceOrder:user:info")
    public Result<UserDTO> get(@PathVariable("id") Long id){
        UserDTO data = userService.get(id);
        data.setRoleIdList(roleUserService.getRoleIdList(data.getId()));

        return new Result<UserDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("serviceOrder:user:save")
    public Result save(@RequestBody UserDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        userService.save(dto);
        userService.saveOrUpdateUserRole(dto.getId(),dto.getRoleIdList());

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("serviceOrder:user:update")
    public Result update(@RequestBody UserDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        userService.update(dto);
        userService.saveOrUpdateUserRole(dto.getId(),dto.getRoleIdList());

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("serviceOrder:user:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        userService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("serviceOrder:user:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<UserDTO> list = userService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, UserExcel.class);
    }

    @PostMapping("importUserByExcel")
    @ApiOperation("导入")
    @LogOperation("导入")
//    @RequiresPermissions("ops:assets:export")
    public Result importUserByExcel(@RequestParam("file") MultipartFile file, HttpServletRequest request) throws Exception {
        if (file.isEmpty()) {
            return new Result<>().error(ErrorCode.UPLOAD_FILE_EMPTY);
        }
        String file_path = QRPath + file.getOriginalFilename();
        File filePath = new File(file_path);
        if (!filePath.getParentFile().exists()) {
            filePath.mkdirs();
        }
        try {
            file.transferTo(filePath);
        } catch (IllegalStateException | IOException e) {
            e.printStackTrace();
        }
        ImportParams params_import = new ImportParams();
        params_import.setHeadRows(1);
        params_import.setTitleRows(0);
        String[] importFields = {"用户名","手机号","生日","地址_地区","详细地址"};
        params_import.setImportFields(importFields);
        params_import.setNeedVerify(true);
        try {
//        List<UserExcel> list = ExcelImportUtil.importExcel(filePath, UserExcel.class, params_import);
            ExcelImportResult<UserExcel> result = null;
            result = ExcelImportUtil.importExcelMore(filePath, UserExcel.class, params_import);
            if (!result.isVerfiyFail()) {
                List<UserExcel> list = result.getList();
                for (UserExcel userExcel : list) {

                    userService.save(ConvertUtils.sourceToTarget(userExcel,UserDTO.class));
                }
                return new Result().ok("成功");
            }else{
                return new Result().error("上传的表格内容存在空值和错误！");
            }
        } catch (Exception e) {
//            throw new IOException(e.getMessage());
            return new Result().error(e.getMessage());
        }
    }

}
